@@ -1,6 +1,6 @@
 # vim:set fileencoding=utf-8:
 # GNU Solfege - free ear training software
-# Copyright (C) 2010  Tom Cato Amundsen
+# Copyright (C) 2010, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,14 +24,15 @@ from __future__ import absolute_import
 # noteheads. So I think it should be given the created NoteHeadEngravers
 # and then shift the notes.
 
+import copy
 import re
 import weakref
 
 from solfege.mpd import _exceptions
 from solfege.mpd import const
-from solfege.mpd import Rat
-from solfege.mpd import Duration
-from solfege.mpd import MusicalPitch
+from solfege.mpd.rat import Rat
+from solfege.mpd.duration import Duration
+from solfege.mpd.musicalpitch import MusicalPitch
 
 #FIXME duplicate of code in Lexer
 re_melodic = re.compile(r"""(?x)
@@ -39,7 +40,7 @@ re_melodic = re.compile(r"""(?x)
                              (?P<octave>[',]*))
                              (?P<len>[\d]*)
                              (?P<dots>\.*)""", re.UNICODE)
-    
+
 class UnknownClefException(_exceptions.MpdException):
     def __init__(self, clef):
         _exceptions.MpdException.__init__(self)
@@ -94,23 +95,22 @@ class Clef(object):
     def steps_to_ylinepos(self, steps):
         return 7-self.clefdata[self.m_name][self.POS] - steps + self.m_octaviation
     def an_to_ylinepos(self, an):
-        def notename_to_ylinepos(n, clef):
+        def notename_to_ylinepos(n):
             n = MusicalPitch.new_from_notename(n)
-            i = n.steps()
-            return self.steps_to_ylinepos(i)
+            return self.steps_to_ylinepos(n.steps())
         if an[-2:] == 'es':
             l = 3
             h = -3
         else:
             l = 1
             h = -5
-        i = notename_to_ylinepos(an, self)
+        i = notename_to_ylinepos(an)
         while i > l:
             an = an + "'"
-            i =  notename_to_ylinepos(an, self)
+            i =  notename_to_ylinepos(an)
         while i < h:
             an = an + ","
-            i = notename_to_ylinepos(an, self)
+            i = notename_to_ylinepos(an)
         return i
 
 class TimeSignature(object):
@@ -151,12 +151,14 @@ class HasParent(object):
                     self.w_score = w().w_parent().w_parent
             except AttributeError:
                 pass
-                
+
 
 class MusicElement(object):
     def __init__(self, duration):
         assert isinstance(duration, Duration)
         self.m_duration = duration
+    def __repr__(self):
+        return unicode(self.__class__)[:-2].replace("<class '", '<') + u" %s at %s>" % (self.m_duration.as_mpd_string(), hex(id(self)))
 
 class Note(MusicElement):
     class Exception(Exception):
@@ -165,6 +167,10 @@ class Note(MusicElement):
         n = Note(self.m_musicalpitch.clone(), self.m_duration.clone())
         n.m_tieinfo = self.m_tieinfo
         return n
+    def __eq__(self, other):
+        assert isinstance(other, Note)
+        return (self.m_musicalpitch == other.m_musicalpitch and
+                self.m_duration == other.m_duration)
     def __init__(self, musicalpitch, duration):
         MusicElement.__init__(self, duration)
         assert isinstance(musicalpitch, MusicalPitch)
@@ -216,7 +222,7 @@ class Stem(list):
         assert isinstance(elemlist, list)
         assert stemdir in (const.UP, const.DOWN, const.BOTH)
         if [x for x in elemlist if x.m_duration != elemlist[0].m_duration]:
-            raise NotEqualLengthException()
+            raise self.NotEqualLengthException()
         list.__init__(self, elemlist)
         self.w_parent = weakref.ref(parent)
         for note in self:
@@ -233,27 +239,39 @@ class Voice(HasParent):
         pass
     class NoChordsInRhythmStaffException(Exception):
         pass
-    def __init__(self, parent):
-        HasParent.__init__(self, parent)
-        # The timelen of the Voice
-        self.m_length = Rat(0, 1)
-        self.m_tdict = {}
+    class NotUnisonException(Exception):
+        pass
     class BarFullException(Exception):
         """
         Exception raised if we try to add a note or rest that is longer than
         the available time in the bar. FIXME: maybe name better, like
         NotEnoughtTimeException or similar.
         """
-        pass
+        def __unicode__(self):
+            return u"There is not enough space left in the bar"
     class NoteDontBelongHere(Exception):
         """
         raised if we try to beam notes that does not belong to this voice.
         """
         pass
+    def __init__(self, parent):
+        HasParent.__init__(self, parent)
+        # The timelen of the Voice
+        self.m_length = Rat(0, 1)
+        self.m_tdict = {}
+    def copy(self, parent):
+        """
+        Return a copy of this Voice object. We make a copy of the dict and
+        the m_length variable, but the dict revers to the same object.
+        """
+        ret = Voice(parent)
+        ret.m_length = Rat(self.m_length.m_num, self.m_length.m_den)
+        ret.m_tdict = self.m_tdict.copy()
+        return ret
     def append(self, elem, stemdir=const.BOTH):
         """
         elem - either a MusicElement or a list of MusicElements
-        
+
         Rests or Skips cannot be put in a list.
 
         If elem is a list of MusicElements (that is notes...), then all
@@ -280,8 +298,7 @@ class Voice(HasParent):
             # IndexError means that self.m_length is after the last bar, this means
             # that the current bar is full.
             bar = self.w_score().add_bar(None)
-        if (self.m_length + elem[0].m_duration.get_rat_value()
-            <= bar.m_timepos + bar.m_timesig.as_rat()):
+        if (self.m_length + elem[0].m_duration.get_rat_value() <= bar.end()):
             if isinstance(elem[0], Note):
                 # We don't have to set w_parent for the Stem and the notes
                 # here, because the Stem constructor does it.
@@ -315,121 +332,109 @@ class Voice(HasParent):
         Delete the element at timepos, move the remaining notes
         in the bar to the left to fill the gap.
         """
-        bar = self.w_score().get_bar_at(timepos)
+        assert timepos in self.m_tdict
+        bp = BarProxy(self, timepos)
+        tp = self.get_timeposes_of(bp.m_bar)[-1]
+        # If the last note in the bar is tied to the next bar, then
+        # we must untie it.
+        if (isinstance(self.m_tdict[tp]['elem'][0], Note)
+            and self.m_tdict[tp]['elem'][0].m_tieinfo in ('go', 'start')):
+            self.untie_next(tp)
+        if isinstance(self.m_tdict[timepos]['elem'][0], Note):
+            if self.is_last(timepos):
+                self.untie_next(timepos)
+            if self.m_tdict[timepos]['elem'][0].m_tieinfo == 'go':
+                self.tie_prev_to_next(timepos)
+            elif self.m_tdict[timepos]['elem'][0].m_tieinfo == 'end':
+                self.untie_prev(timepos)
+            elif self.m_tdict[timepos]['elem'][0].m_tieinfo == 'start':
+                self.untie_next(timepos)
         del self.m_tdict[timepos]
-        self._zap_skips(bar)
-        self._bar_fill_skips(bar)
-    def try_set_elem(self, elem, timepos):
+        bp.remove_skips()
+        bp.repack()
+        bp.fill_skips()
+    def try_set_elem(self, elem, timepos, insert_mode):
         """
         Replace whatever is at timepos with elem.
-        This method expects that there are no skips before timepos.
         Return True if succuessful.
         """
-        # Check that we the elem before is not a Skip
-        idx = sorted(self.m_tdict.keys()).index(timepos)
-        if idx > 0:
-            prev_timepos = sorted(self.m_tdict.keys())[idx - 1]
-        else:
-            prev_timepos = None
-        assert (prev_timepos == None
-                or not isinstance(self.m_tdict[prev_timepos]['elem'][0], Skip))
-        # finished asserts
-        bar = self.w_score().get_bar_at(timepos)
-        self._get_free_time(bar)
+        bp = BarProxy(self, timepos)
         if isinstance(self.m_tdict[timepos]['elem'][0], Skip):
-            if timepos + elem.m_duration.get_rat_value() <= bar.end():
+            if timepos + elem.m_duration.get_rat_value() <= bp.end():
                 if isinstance(elem, Note):
                     stem = Stem(self, [elem], const.UP)
                     self.set_elem(stem, timepos)
                 else:
                     self.set_elem([elem], timepos)
                     elem.w_parent = weakref.ref(self)
-                self._bar_remove_skips(bar)
-                self._bar_fill_skips(bar)
+                bp.remove_skips()
+                bp.repack()
+                bp.fill_skips()
                 return True
         else:
-            # Replace the current note.
-            if self._get_free_time(bar) + self.m_tdict[timepos]['elem'][0].m_duration.get_rat_value() - elem.m_duration.get_rat_value() >= Rat(0, 1):
-                # We have space to add.
-                stem = Stem(self, [elem], const.UP)
-                self.set_elem(stem, timepos)
-                self._bar_remove_skips(bar)
-                self._zap_skips(bar)
-                self._bar_fill_skips(bar)
-                return True
-    def _get_free_time(self, bar):
-        """
-        Return the duration, as a Rat value, on the end of the bar
-        consisting of Rests and Skips.
-        """
-        d = Rat(0, 1)
-        for timepos in reversed(self.get_timeposes_of(bar)):
-            if not isinstance(self.m_tdict[timepos]['elem'][0], (Skip, Rest)):
-                break
-            d += self.m_tdict[timepos]['elem'][0].m_duration.get_rat_value()
-        return d
-    def _bar_remove_skips(self, bar):
-        """
-        Remove the skips from the bar, if any.
-        """
-        for t in self.get_timeposes_of(bar):
-            if isinstance(self.m_tdict[t]['elem'][0], Skip):
-                del self.m_tdict[t]
-    def _bar_fill_skips(self, bar):
-        """
-        Add Skips at the end of the bar, so that it is filled.
-        We assume that any elements already added are placed at
-        the correct timepos.
-        """
-        # nt = short for "next timepos", the timepos to start fill skips to
-        if self.get_timeposes_of(bar):
-            nt = self.get_timeposes_of(bar)[-1]
-            nt = nt + self.m_tdict[nt]['elem'][0].m_duration.get_rat_value()
-        else:
-            # we get here if the bar is empty
-            nt = bar.m_timepos
-        default_skip = Rat(1, 4)
-        # pos within default skip
-        pp =  nt - int (nt / default_skip) * default_skip
-        if pp != Rat(0, 1):
-            # Here we add a skip so that the next column will be X times
-            # default_skip
-            self.set_elem([Skip(Duration.new_from_rat(default_skip - pp))],
-                          nt)
-            nt += (default_skip - pp)
-        # And the we fill the bar with Skips as long as default_skip.
-        while nt < bar.end():
-            self.set_elem([Skip(Duration.new_from_rat(default_skip))], nt)
-            nt += default_skip
-    def _zap_skips(self, bar):
-        """
-        Remove any skips from the bar. Move notes
-        to the left so all remaining time is on the left of the bar.
-        """
-        corrected_timepos = bar.m_timepos
-        mods = {}
-        for t in self.get_timeposes_of(bar):
-            if t != corrected_timepos:
-                mods[corrected_timepos] = t
-            corrected_timepos += self.m_tdict[t]['elem'][0].m_duration.get_rat_value()
-        if mods:
-            # This conditional is true if the elem we are replacing is shorter
-            # than the new one. We must select the correct order to adjust the
-            # timeposes so we don't overwrite values
-            if mods.items()[0][0] < mods.items()[0][1]:
-                keys = sorted(mods.keys())
-            else:
-                keys = reversed(sorted(mods.keys()))
-            for n in keys:
-                assert n not in self.m_tdict
-                self.m_tdict[n] = self.m_tdict[mods[n]]
-                del self.m_tdict[mods[n]]
+            max_free = bp.get_free_time()
+            #flytt til bar-class
+            delta = elem.m_duration.get_rat_value() - self.m_tdict[timepos]['elem'][0].m_duration.get_rat_value()
+            def fix_ties(prev_timepos, cur_timepos, next_timepos):
+                prev_e = self.m_tdict[prev_timepos]['elem'][0] if prev_timepos else None
+                cur_e = self.m_tdict[cur_timepos]['elem'][0]
+                next_e = self.m_tdict[next_timepos]['elem'][0] if next_timepos else None
+                if isinstance(elem, Note):
+                    t = None
+                    if prev_timepos and isinstance(prev_e, Note) and prev_e.m_tieinfo in ('go', 'start'):
+                        t = 'end'
+                    if isinstance(next_e, Note) and next_e.m_tieinfo in ('go', 'end'):
+                        if t == 'end':
+                            t = 'go'
+                        else:
+                            t = 'start'
+                    cur_e.m_tieinfo = t
+                else:
+                    assert isinstance(elem, Rest)
+                    if prev_timepos and isinstance(prev_e, Note):
+                        if prev_e.m_tieinfo == 'go':
+                            prev_e.m_tieinfo = 'end'
+                        elif prev_e.m_tieinfo == 'start':
+                            prev_e.m_tieinfo = None
+                    if isinstance(next_e, Note):
+                        if next_e.m_tieinfo == 'go':
+                            next_e.m_tieinfo = 'start'
+                        elif next_e.m_tieinfo == 'end':
+                            next_e.m_tieinfo = None
+
+            if insert_mode:
+                if max_free >= elem.m_duration.get_rat_value():
+                    tmp_timepos = timepos + Rat(1, 1000000)
+                    self.m_tdict[tmp_timepos] = self.m_tdict[timepos]
+                    del self.m_tdict[timepos]
+                    bp.remove_trailing(elem.m_duration.get_rat_value())
+                    stem = Stem(self, [elem], const.UP)
+                    self.set_elem(stem, timepos)
+                    fix_ties(self.get_prev_timepos(timepos), timepos, tmp_timepos)
+                    bp.remove_skips()
+                    bp.repack()
+                    bp.fill_skips()
+                    return True
+            if not insert_mode:
+                if (max_free >= delta):
+                    # We have space to add.
+                    # Delete skips (and rests) from the end of the bar
+                    # until we have enough space to add the elem.
+                    if delta > Rat(0, 1):
+                        bp.remove_trailing(delta)
+                    stem = Stem(self, [elem], const.UP)
+                    self.set_elem(stem, timepos)
+                    fix_ties(self.get_prev_timepos(timepos), timepos, self.get_next_timepos(timepos))
+                    bp.remove_skips()
+                    bp.repack()
+                    bp.fill_skips()
+                    return True
     def fill_with_skips(self):
         """
         Fill the voice with skips. We assume the voice is empty.
         """
         for bar in self.w_score().m_bars:
-            self._bar_fill_skips(bar)
+            bar.fill_skips(self)
     def get_timeposes_of(self, bar):
         """
         Return a sorted list of all timeposes in bar in this Voice
@@ -444,10 +449,10 @@ class Voice(HasParent):
             start_i = 0
             while start_i < len(v) and v[start_i] < bar.m_timepos:
                 start_i += 1
-        if start_i >= len(v) or v[start_i] >= bar.m_timepos + bar.m_timesig.as_rat():
+        if start_i >= len(v) or v[start_i] >= bar.end():
             start_i = None
         try:
-            end_i = v.index(bar.m_timepos + bar.m_timesig.as_rat())
+            end_i = v.index(bar.end())
         except ValueError:
             end_i = None
         if start_i == None:
@@ -456,6 +461,23 @@ class Voice(HasParent):
             return v[start_i:end_i]
         else:
             return v[start_i:]
+    def get_time_pitch_list(self, bpm):
+        """
+        Return a list of tuples (pitch, duration-in-seconds) of the tones
+        and rests in the voice. -1 is used for pitch for rests.
+        """
+        ret = []
+        for timepos in sorted(self.m_tdict.keys()):
+            # stem is a Stem or [Rest]
+            stem = self.m_tdict[timepos]['elem']
+            if len(stem) != 1:
+                raise Voice.NotUnisonException()
+            dur = float(stem[0].m_duration.get_rat_value()) * bpm[1] / bpm[0] * 60
+            if isinstance(stem[0], Note):
+                ret.append((stem[0].m_musicalpitch.semitone_pitch(), dur))
+            elif isinstance(stem[0], Rest):
+                ret.append((-1, dur))
+        return ret
     def beam(self, notes):
         """
         Notes is a list of Note objects, once Note object from each stem
@@ -494,7 +516,7 @@ class Voice(HasParent):
             self.tie([self.m_tdict[timepos]['elem'][0],
                       self.m_tdict[next_timepos]['elem'][0]])
             return True
-    def untie_timepos(self, timepos):
+    def untie_next(self, timepos):
         """
         Return True if we removed the tie from timepos to the note after.
         Return False if not.
@@ -519,6 +541,33 @@ class Voice(HasParent):
         else:
             return False
         return True
+    def untie_prev(self, timepos):
+        """
+        Remove True if we removed a tie from the prev note to this.
+        Return False if not.
+        """
+        prev_timepos = self.get_prev_timepos(timepos)
+        if self.m_tdict[timepos]['elem'][0].m_tieinfo == 'end':
+            if self.m_tdict[prev_timepos]['elem'][0].m_tieinfo == 'start':
+                self.m_tdict[prev_timepos]['elem'][0].m_tieinfo = None
+            elif self.m_tdict[prev_timepos]['elem'][0].m_tieinfo == 'go':
+                self.m_tdict[prev_timepos]['elem'][0].m_tieinfo = 'end'
+            else:
+                return False
+            self.m_tdict[timepos]['elem'][0].m_tieinfo = None
+        elif self.m_tdict[timepos]['elem'][0].m_tieinfo == 'go':
+            if self.m_tdict[prev_timepos]['elem'][0].m_tieinfo == 'start':
+                self.m_tdict[prev_timepos]['elem'][0].m_tieinfo = None
+            elif self.m_tdict[prev_timepos]['elem'][0].m_tieinfo == 'go':
+                self.m_tdict[prev_timepos]['elem'][0].m_tieinfo = 'end'
+            else:
+                return False
+            self.m_tdict[timepos]['elem'][0].m_tieinfo = 'start'
+    def tie_prev_to_next(self, timepos):
+        assert self.m_tdict[timepos]['elem'][0].m_tieinfo == 'go'
+        assert self.m_tdict[self.get_prev_timepos(timepos)]['elem'][0].m_tieinfo in ('start', 'go')
+        assert self.m_tdict[self.get_next_timepos(timepos)]['elem'][0].m_tieinfo in ('end', 'go')
+        self.m_tdict[timepos]['elem'][0].m_tieinfo = None
     def tie(self, notes):
         """
         The notes should be in sequence on this Voice. Behaviour if not
@@ -591,6 +640,16 @@ class Voice(HasParent):
             return bar.m_timepos == self.m_length
         except IndexError:
             return True
+    def get_prev_timepos(self, timepos):
+        """
+        Return the previous timepos. Return None if this is the first
+        timepos in the voice.
+        """
+        assert timepos in self.m_tdict
+        v = sorted(self.m_tdict.keys())
+        i = v.index(timepos)
+        if i > 0:
+            return v[i - 1]
     def get_next_timepos(self, timepos):
         """
         Return the next timepos. Return None if this is the last timepos
@@ -599,11 +658,9 @@ class Voice(HasParent):
         assert timepos in self.m_tdict
         v = sorted(self.m_tdict.keys())
         i = v.index(timepos)
-        if i + 1 == len(v):
-            return None
-        return v[i + 1]
+        if i +1 < len(v):
+            return v[i + 1]
     def get_timelist(self):
-        curpos = Rat(0, 1)
         retval = []
         for timepos in sorted(self.m_tdict.keys()):
             if isinstance(self.m_tdict[timepos]['elem'][0], Rest):
@@ -624,6 +681,13 @@ class Voice(HasParent):
                   nlen])
                 nlen = None
         return retval
+    def is_last(self, timepos):
+        """
+        Return a bool telling if the elem at timepos is the last in the
+        bar and fill it, so that the next elem has to be the first in the
+        next bar.
+        """
+        return self.m_tdict[timepos]['elem'][0].m_duration.get_rat_value() + timepos == self.w_score().get_bar_at(timepos).end()
     def __getitem__(self, idx):
         v = sorted(self.m_tdict)
         return self.m_tdict[v[idx]]
@@ -634,7 +698,7 @@ class Bar(object):
         Time signature is the time signature of the bar. It does not
         necessarily mean that we will engrave a time signature on the
         score. All bar objects need to know their length.
-        
+
         timepos is the time of the first music in the bar. So timepos
         + timesig will be the position of the next bar.
         """
@@ -644,10 +708,142 @@ class Bar(object):
         self.m_timepos = timepos
     def end(self):
         return self.m_timepos + self.m_timesig.as_rat()
+    def fill_skips(self, voice):
+        """
+        Add Skips at the end of the bar, so that it is filled.
+        We assume that any elements already added are placed at
+        the correct timepos.
+        """
+        # nt = short for "next timepos", the timepos to start fill skips to
+        if voice.get_timeposes_of(self):
+            nt = voice.get_timeposes_of(self)[-1]
+            nt = nt + voice.m_tdict[nt]['elem'][0].m_duration.get_rat_value()
+        else:
+            # we get here if the bar is empty
+            nt = self.m_timepos
+        default_skip = Rat(1, 4)
+        # pos within default skip
+        pp =  nt - int (nt / default_skip) * default_skip
+        if pp != Rat(0, 1):
+            # Here we add a skip so that the next column will be X times
+            # default_skip
+            voice.set_elem([Skip(Duration.new_from_rat(default_skip - pp))],
+                          nt)
+            nt += (default_skip - pp)
+        # And the we fill the bar with Skips as long as default_skip.
+        while nt < self.end():
+            voice.set_elem([Skip(Duration.new_from_rat(default_skip))], nt)
+            nt += default_skip
+    def get_free_time(self, voice):
+        """
+        Return the duration, as a Rat value, on the end of the bar
+        consisting of Rests and Skips.
+        """
+        d = Rat(0, 1)
+        for timepos in reversed(voice.get_timeposes_of(self)):
+            if not isinstance(voice.m_tdict[timepos]['elem'][0], (Skip, Rest)):
+                break
+            d += voice.m_tdict[timepos]['elem'][0].m_duration.get_rat_value()
+        return d
+    def pop_last_elem(self, voice):
+        """
+        Remove the last element form the voice, and return its
+        duration as a Rat.
+        """
+        timepos = voice.get_timeposes_of(self)[-1]
+        ret = voice.m_tdict[timepos]['elem'][0].m_duration.get_rat_value()
+        del voice.m_tdict[timepos]
+        return ret
+    def remove_skips(self, voice):
+        """
+        Remove the skips from the bar, if any.
+        Leave bar in inconsistent state. Need bar.repack or similar after
+        calling this method.
+        """
+        for t in voice.get_timeposes_of(self):
+            if isinstance(voice.m_tdict[t]['elem'][0], Skip):
+                del voice.m_tdict[t]
+    def remove_trailing(self, voice, duration):
+        """
+        Remove elements from the end of the bar, until their duration
+        is a least 'duration' long.
+        """
+        assert isinstance(duration, Rat)
+        total = Rat(0, 1)
+        while total < duration:
+            total += self.pop_last_elem(voice)
+    def repack(self, voice):
+        """
+        Call this method to cleanup m_tdict for the bar after keys
+        from m_tict have ben deleted. Example before bar.repack:
+        0/4: Note
+        1/4: Rest
+        1/2: Note
+
+        then del m_tdict[Rat(1, 4)] make the content of tdict look like this:
+        0/4: Note
+        1/2: Note
+
+        Calling bar.repack will repack is like this:
+        0/4: Note
+        1/4: Note
+
+        NOTE: This method does not add or remove any elements, including Skips
+        """
+        timeposes = voice.get_timeposes_of(self)
+        if timeposes:
+            last_is_last = voice.is_last(timeposes[-1])
+        else:
+            last_is_last = None
+        corrected_timepos = self.m_timepos
+        mods = {}
+        for t in voice.get_timeposes_of(self):
+            if t != corrected_timepos:
+                mods[corrected_timepos] = t
+            corrected_timepos += voice.m_tdict[t]['elem'][0].m_duration.get_rat_value()
+            if corrected_timepos > self.end():
+                raise voice.BarFullException()
+        if mods:
+            # This conditional is true if the elem we are replacing is shorter
+            # than the new one. We must select the correct order to adjust the
+            # timeposes so we don't overwrite values
+            if mods.items()[0][0] < mods.items()[0][1]:
+                keys = sorted(mods.keys())
+            else:
+                keys = reversed(sorted(mods.keys()))
+            for n in keys:
+                assert n not in voice.m_tdict
+                voice.m_tdict[n] = voice.m_tdict[mods[n]]
+                del voice.m_tdict[mods[n]]
+        # If the last note is not filling the bar any more, then we must
+        # remove the tie.
+        if last_is_last:
+            timeposes = voice.get_timeposes_of(self)
+            if (timeposes and not voice.is_last(timeposes[-1])
+                and isinstance(voice.m_tdict[timeposes[-1]]['elem'][0], Note)
+                and voice.m_tdict[timeposes[-1]]['elem'][0].m_tieinfo in ('go', 'start')):
+                voice.untie_next(timeposes[-1])
     def __repr__(self):
-        return "<Bar %i/%i %s at %s>" % (self.m_timesig.m_num,
+        return "<%s %i/%i %s at %s>" % (str(self.__class__).split(".")[-1][:-2], self.m_timesig.m_num,
             self.m_timesig.m_den, self.m_timepos, hex(id(self)))
 
+class PartialBar(Bar):
+    def __init__(self, duration, timesig, timepos):
+        Bar.__init__(self, timesig, timepos)
+        assert isinstance(duration, Duration)
+        self.m_duration = duration
+    def end(self):
+        return self.m_timepos + self.m_duration.get_rat_value()
+
+class BarProxy(object):
+    def __init__(self, voice, timepos):
+        self.m_voice = voice
+        self.m_bar = voice.w_parent().w_parent().get_bar_at(timepos)
+    def __getattr__(self, attr):
+        if attr == 'end':
+            return self.m_bar.end
+        return lambda *f: getattr(self.m_bar, attr)(self.m_voice, *f)
+
 class _StaffCommon(HasParent):
     """
     A voice is added to the staff when it is created.
@@ -657,7 +853,15 @@ class _StaffCommon(HasParent):
         HasParent.__init__(self, parent)
         self.m_voices = []
         self.add_voice()
+        # I think the only things stored in Staff.m_tdict are "clef" and
+        # "keysig". We don't store time signature changes where, since
+        # Score.m_bars take care about that.
         self.m_tdict = {}
+    def copy(self, parent):
+        staff = self.__class__(parent)
+        staff.m_voices = [v.copy(staff) for v in self.m_voices]
+        staff.m_tdict = self.m_tdict.copy()
+        return staff
     def add_voice(self):
         self.m_voices.append(Voice(self))
         self.w_score().create_shortcuts()
@@ -687,11 +891,10 @@ class _StaffCommon(HasParent):
                         data[endpos] = {'start': set(), 'end': set()}
                     data[endpos]['end'].add(voice_idx)
         retval = []
-        start_time = None
         v = sorted(data)[:]
         ppos = Rat(0, 1)
         voices = set()
-        for idx, timepos in enumerate(v):
+        for timepos in v:
             if data[timepos]['start'] and data[timepos]['end']:
                 for voice in data[timepos]['end']:
                     voices.remove(voice)
@@ -714,6 +917,10 @@ class _StaffCommon(HasParent):
                 if not voices:
                     retval.append([True, timepos - ppos])
         return retval
+    def set_property(self, timepos, name, value):
+        d = self.m_tdict.setdefault(timepos, {})
+        properties = d.setdefault('properties', {})
+        properties[name] = value
 
 class Staff(_StaffCommon):
     def __init__(self, parent):
@@ -740,7 +947,7 @@ class RhythmStaff(_StaffCommon):
         _StaffCommon.__init__(self, parent)
     def add_voice(self):
         if len(self.m_voices) == 1:
-            raise OnlyOneVoiceException()
+            raise self.OnlyOneVoiceException()
         _StaffCommon.add_voice(self)
     def set_key_signature(self, keysig, timepos):
         """
@@ -750,9 +957,22 @@ class RhythmStaff(_StaffCommon):
 
 
 class Score(object):
+    class ConcatException(Exception):
+        pass
+    class StaffCountException(ConcatException):
+        pass
+    class StaffTypeException(ConcatException):
+        pass
+    class VoiceCountException(ConcatException):
+        pass
     def __init__(self):
         self.m_staffs = []
         self.m_bars = []
+    def copy(self):
+        score = Score()
+        score.m_staffs = [s.copy(score) for s in self.m_staffs]
+        score.m_bars = copy.deepcopy(self.m_bars)
+        return score
     def add_staff(self, staff_class=Staff):
         self.m_staffs.append(staff_class(self))
         self.create_shortcuts()
@@ -766,29 +986,46 @@ class Score(object):
             setattr(self, "staff%i" % (staff_idx + 1), staff)
             for voice_idx, voice in enumerate(staff.m_voices):
                 setattr(self, "voice%i%i" % (staff_idx + 1, voice_idx + 1), voice)
+    def _get_new_bar_timepos(self):
+        """
+        Return the timepos where the next bar will be added.
+        """
+        if self.m_bars:
+            return self.m_bars[-1].end()
+        return Rat(0, 1)
+    def _get_new_bar_timesig(self, timesig):
+        """
+        Return the time signature the next bar will get.
+        """
+        if timesig:
+            return timesig
+        elif self.m_bars:
+            return self.m_bars[-1].m_timesig
+        else:
+            return TimeSignature(4, 4)
     def add_bar(self, timesig):
         """
         If timesig is None, then we use the same timesig as the last bar.
         Return the added bar.
         """
-        if self.m_bars:
-            c = self.m_bars[-1].m_timepos + self.m_bars[-1].m_timesig.as_rat()
-        else:
-            c = Rat(0, 1)
-        if timesig:
-            self.m_bars.append(Bar(timesig, c))
-        elif self.m_bars:
-            self.m_bars.append(Bar(self.m_bars[-1].m_timesig, c))
-        else:
-            self.m_bars.append(Bar(TimeSignature(4, 4), c))
+        self.m_bars.append(Bar(
+            self._get_new_bar_timesig(timesig), self._get_new_bar_timepos()))
+        return self.m_bars[-1]
+    def add_partial_bar(self, duration, timesig):
+        """
+        Set to the duration of the pickup bar if we want one.
+        This must be called before bars are added with Score.add_bar
+        """
+        self.m_bars.append(PartialBar(duration,
+            self._get_new_bar_timesig(timesig), self._get_new_bar_timepos()))
         return self.m_bars[-1]
     def get_bar_at(self, timepos):
         """
         Return the bar timepos is within. Raise IndexError if timepos
         is after the last bar.
         """
-        for idx, bar in enumerate(self.m_bars):
-            if bar.m_timepos <= timepos <bar.m_timepos + bar.m_timesig.as_rat():
+        for bar in self.m_bars:
+            if bar.m_timepos <= timepos < bar.end():
                 return bar
             if bar.m_timepos > timepos:
                 return IndexError(timepos)
@@ -810,11 +1047,10 @@ class Score(object):
                                 data[endpos] = {'start': set(), 'end': set()}
                             data[endpos]['end'].add((staff_idx, voice_idx))
         retval = []
-        start_time = None
         v = sorted(data)[:]
         ppos = Rat(0, 1)
         voices = set()
-        for idx, timepos in enumerate(v):
+        for timepos in v:
             if data[timepos]['start'] and data[timepos]['end']:
                 for voice in data[timepos]['end']:
                     voices.remove(voice)
@@ -840,4 +1076,72 @@ class Score(object):
                     retval.append([True, timepos - ppos])
                     ppos = timepos
         return retval
-
+    @staticmethod
+    def concat(s1, s2):
+        """
+        Concatenate the two scores, and return a new score. Both scores
+        need to have the exact same staff and voice layout.
+        """
+        assert isinstance(s1, Score)
+        assert isinstance(s2, Score)
+        if len(s1.m_staffs) != len(s2.m_staffs):
+            raise Score.StaffCountException()
+        if [type(x) for x in s1.m_staffs] != [type(x) for x in s2.m_staffs]:
+            raise Score.StaffTypeException()
+        for st1, st2 in zip(s1.m_staffs, s2.m_staffs):
+            if len(st1.m_voices) != len(st2.m_voices):
+                raise Score.VoiceCountException()
+        ret = s1.copy()
+        if not s1.m_staffs:
+            return s1
+        # do the adding
+        for bar in s2.m_bars:
+            bar.m_timepos = ret.m_bars[-1].end()
+            ret.m_bars.append(bar)
+        ret.create_shortcuts()
+        if s1.m_bars:
+            start = s1.m_bars[-1].end()
+        else:
+            start = Rat(0, 1)
+        s2.create_shortcuts() # FIXME why?
+        for staff_idx, staff in enumerate(s1.m_staffs):
+            for voice_idx in range(len(staff.m_voices)):
+                for k in s2.m_staffs[staff_idx].m_voices[voice_idx].m_tdict:
+                    ret.m_staffs[staff_idx].m_voices[voice_idx].m_tdict[k + start] = s2.m_staffs[staff_idx].m_voices[voice_idx].m_tdict[k]
+        return ret
+    @staticmethod
+    def concat2(s1, s2):
+        """
+        Return a new Score object concatenating the two scores. This is
+        intended return value is intended for playback only, since the
+        staffs placed below each other. So the first score will have empty
+        bars at the end, and the last score will have empty bars at the
+        beginning.
+        """
+        assert isinstance(s1, Score)
+        assert isinstance(s2, Score)
+        ret = s1.copy()
+        if s1.m_bars:
+            start = s1.m_bars[-1].end()
+        else:
+            start = Rat(0, 1)
+        for bar in s2.m_bars:
+            ret.m_bars.append(Bar(bar.m_timesig, ret.m_bars[-1].end()))
+        for staff_idx, staff in enumerate(s2.m_staffs):
+            ret.add_staff(staff_class=staff.__class__)
+            for k in staff.m_tdict:
+                ret.m_staffs[-1].m_tdict[start + k] = staff.m_tdict[k]
+            for voice_idx in range(len(staff.m_voices)):
+                if voice_idx != 0:
+                    ret.m_staffs[-1].add_voice()
+                # This line make the music from sc2 continue after the
+                # point where the music from sc1 ends.
+                ret.m_staffs[-1].m_voices[-1].m_length = s1.m_bars[-1].end()
+                for elem in s2.m_staffs[staff_idx].m_voices[voice_idx]:
+                    ret.m_staffs[-1].m_voices[-1].append(elem['elem'])
+        ret.create_shortcuts()
+        return ret
+    def __deepcopy__(self, memo):
+        ret = Score()
+        ret.m_bars = copy.deepcopy(self.m_bars)
+        return ret
